@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -36,6 +36,7 @@
 #include "config.h"
 #include "emu.h"
 #include "cpu.h"
+#include "bios.h"
 #include "shared.h"
 #include "dpmi.h"
 
@@ -333,7 +334,7 @@ void mhp_exit_intercept(int errcode)
 unsigned int mhp_debug(unsigned int code, unsigned int parm1, unsigned int parm2)
 {
   int rtncd = 0;
-  extern void bios_f000(), DBGload(), DBGload_CSIP(), DBGload_parblock();
+  extern void DBGload(), DBGload_CSIP(), DBGload_parblock();
 #if 0
   return rtncd;
 #endif
