@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -62,6 +62,7 @@
 #include "emu.h"
 #include "cpu.h"
 #include "dpmi.h"
+#include "utilities.h"
 
 #define MHP_PRIVATE
 #include "mhpdbg.h"
@@ -91,7 +92,6 @@ extern int  dis_8086(unsigned int, const unsigned char *,
 
 /* prototypes */
 static void* mhp_getadr(unsigned char *, unsigned int *, unsigned int *, unsigned int *);
-static int mhp_parse(char*, char *[]);
 static void mhp_regs  (int, char *[]);
 static void mhp_r0    (int, char *[]);
 static void mhp_dis   (int, char *[]);
@@ -214,6 +214,7 @@ static const char help_page[]=
   "bl                     list active breakpoints\n"
   "bplog/bclog regex      set/clear breakpoint on logoutput using regex\n"
   "rusermap org fn        read microsoft linker format .MAP file 'fn'\n"
+  "rmapfile [file]        (re)read a dosemu.map ('nm' format) file\n"
   "                       code origin = 'org'.\n"
   "ldt sel lines          dump ldt starting at selector 'sel' for 'lines'\n"
   "log [flags]            get/set debug-log flags (e.g 'log +M-k')\n"
@@ -388,16 +389,23 @@ static void mhp_rusermap(int argc, char *argv[])
 
 static void mhp_rmapfile(int argc, char *argv[])
 {
+  PRIV_SAVE_AREA
   FILE * ifp;
   unsigned char bytebuf[IBUFS];
   unsigned long a1;
+  char *map_fname = DOSEMU_MAP_PATH;
 
-  ifp = fopen(DOSEMU_MAP_PATH, "r");
+  if (argc >= 2) {
+    map_fname = argv[1];
+  }
+  enter_priv_off();
+  ifp = fopen(map_fname, "r");
+  leave_priv_setting();
   if (!ifp) {
-     mhp_printf("unable to open map file %s\n", DOSEMU_MAP_PATH);
+     mhp_printf("unable to open map file %s\n", map_fname);
      return;
   }
-  mhp_printf("Reading map file %s\n", DOSEMU_MAP_PATH);
+  mhp_printf("Reading map file %s\n", map_fname);
   last_symbol = 0;
   while (last_symbol < MAXSYM) {
      if(!fgets(bytebuf, 100, ifp))
@@ -774,14 +782,14 @@ static void mhp_disasm(int argc, char * argv[])
 #endif
 
    if (IN_DPMI) {
-     def_size = dpmi_mhp_get_selector_size(seg);
+     def_size = (dpmi_mhp_get_selector_size(seg)? 3:0);
      segmented =1;
    }
    else {
-     if (seekval < (a20 ? 0x10fff0 : 0x100000)) def_size = linmode;
+     if (seekval < (a20 ? 0x10fff0 : 0x100000)) def_size = (linmode? 3:0);
      else if (lastu[0] == '#')
         def_size = 0;
-     else def_size = 1;
+     else def_size = 3;
    }
    rc=0;
    buf = (unsigned char *) seekval;
@@ -1252,39 +1260,6 @@ static void mhp_bpload(int argc, char * argv[])
    return;
 }
 
-static int mhp_parse(char *s, char* argvx[])
-{
-   int mode = 0;
-   int argcx = 0;
-   char delim = 0;
-
-   for ( ; *s; s++) {
-      if (!mode) {
-         if (*s > ' ') {
-            mode = 1;
-            argvx[argcx++] = s;
-            switch (*s) {
-              case '"':
-              case '\'':
-                delim = *s;
-                mode = 2;
-            }
-            if (argcx >= MAXARG)
-               break;
-         }
-      } else if (mode == 1) {
-         if (*s <= ' ') {
-            mode = 0;
-            *s = 0x00;
-         }
-      } else {
-         if (*s == delim) mode = 1;
-      }
-   }
-   argvx[argcx] = 0;
-   return(argcx);
-}
-
 static void mhp_regs(int argc, char * argv[])
 {
   unsigned long newval;
@@ -1463,27 +1438,7 @@ void mhp_modify_eip(int delta)
 
 void mhp_cmd(const char * cmd)
 {
-   int argc1;
-   char * argv1[MAXARG];
-   char tmpcmd[250];
-   void (*cmdproc)(int, char *[]);
-   const struct cmd_db * cmdp;
-
-   strcpy(tmpcmd, cmd);
-   argc1 = mhp_parse(tmpcmd, argv1);
-   if (argc1 < 1)
-      return;
-   for (cmdp = cmdtab, cmdproc = NULL; cmdp->cmdproc; cmdp++) {
-      if (!memcmp(cmdp->cmdname, argv1[0], strlen(argv1[0])+1)) {
-         cmdproc = cmdp->cmdproc;
-         break;
-      }
-   }
-   if (!cmdproc) {
-      mhp_printf("Command %s not found\n", argv1[0]);
-      return;
-   }
-   (*cmdproc)(argc1, argv1);
+   call_cmd(cmd, MAXARG, cmdtab, mhp_printf);
 }
 
 static void mhp_print_ldt(int argc, char * argv[])
