@@ -52,6 +52,8 @@
 int X_change_config(unsigned, void *);
 #endif
 
+extern void pci_bios(void);
+
 /*
    This flag will be set when doing video routines so that special
    access can be given
@@ -376,6 +378,18 @@ static int dos_helper(void)
     mfs_inte6();
     return 1;
 
+  case DOS_HELPER_DOSC:
+    if (HI(ax) == 0xdc) {
+      /* install check and notify */
+      if (!dosc_interface()) return 0;
+      running_DosC = LWORD(ebx);
+      return 1;
+    }
+    if (running_DosC) {
+      return dosc_interface();
+    }
+    return 0;
+
   case DOS_HELPER_EMS_HELPER:
     ems_helper();
     return 1;
@@ -1082,19 +1096,7 @@ Return: nothing
     break;
  
   case 0xb1:			/* Intel PCI BIOS v 2.0c */
-    switch (LO(ax)) {
-      case 1: case 0x81:	/* installation check */
-        NOCARRY;		/* AH==0 if installed */
-        break;
-      case 2: case 0x82:	/* find PCI device */
-        g_printf("FIND PCI device %d id=%04x vend=%04x\n", LWORD(esi),
-        	LWORD(ecx), LWORD(edx));
-      case 3: case 0x83:	/* find PCI class code */
-	HI(ax) = 0x86;		/* not found */
-      default:
-        CARRY;
-        break;
-    }
+      pci_bios();
     break;
 
   default:
@@ -1528,7 +1530,13 @@ static void redirect_devices()
  */
 static int redir_it()
 {
-  static struct vm86_regs save_regs;
+  /*
+   * Declaring the following struct volatile works around an EGCS bug
+   * (at least up to egcs-2.91.66). Otherwise the line below marked
+   * with '###' will modify the *old* (saved) struct too.
+   * -- sw
+   */
+  volatile static struct vm86_regs save_regs;
   static unsigned x0, x1, x2, x3, x4;
   unsigned u;
 
@@ -1539,10 +1547,20 @@ static int redir_it()
   switch(redir_state) {
     case 1:
       if(HI(ax) == 0x3d) {
+        /*
+         * FreeDOS will get confused by the following calling sequence (e.i. it
+         * is not reentrant 'enough'. So we will abort here - it cannot use a
+         * redirector anyway.
+         * -- sw
+         */
+        if (running_DosC) {
+          ds_printf("INT21: FreeDOS detected - no check for redirector\n");
+          return redir_state = 0;
+        }
         save_regs = REGS;
         redir_state = 2;
         LWORD(eip) -= 2;
-        LWORD(eax) = 0x5200;
+        LWORD(eax) = 0x5200;		/* ### , see above EGCS comment! */
         default_interrupt(0x21);
         ds_printf("INT21 +1 (%d) at %04x:%04x: AX=%04x, BX=%04x, CX=%04x, DX=%04x, DS=%04x, ES=%04x\n",
           redir_state, LWORD(cs), LWORD(eip), LWORD(eax), LWORD(ebx), LWORD(ecx), LWORD(edx), LWORD(ds), LWORD(es));
