@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -36,7 +36,7 @@
 
 #include <features.h>
 #include <sys/time.h>
-#if __GLIBC__ > 1
+#if GLIBC_VERSION_CODE >= 2000
 #include <sys/kd.h>
 #else
 #include <linux/kd.h>
@@ -52,11 +52,14 @@
 
 #include "keyb_server.h"
 
+#undef  DEBUG_PIT
 
 #ifdef MONOTON_MICRO_TIMING
-extern long   pic_itime[33];
+extern hitimer_t pic_itime[33];
 #endif /* MONOTON_MICRO_TIMING */
 
+extern hitimer_t t_vretrace;	/* see env/video/miscemu.c */
+
 pit_latch_struct pit[PIT_TIMERS];   /* values of 3 PIT counters */
 
 static u_long timer_div;          /* used by timer int code */
@@ -213,7 +216,7 @@ void do_sound(Bit16u period)
 static void pit_latch(int latch)
 {
   hitimer_u cur_time;
-  u_long         ticks;
+  u_long ticks=0;
   pit_latch_struct *p = &pit[latch];
 
   /* check for special 'read latch status' mode */
@@ -291,23 +294,26 @@ static void pit_latch(int latch)
     /* mode 6 -- ??? */
     /* mode 3 -- square-wave generator, countdown by 2 */
     /* mode 7 -- ??? */
-      pic_sys_time = cur_time.t.tl;	/* modulo 2^32 */
+      pic_sys_time = cur_time.td;	/* full counter */
+    /* NEVER is a special invalid value... skip it if found */
       pic_sys_time += (pic_sys_time == NEVER);
 
       if (latch == 0) {
 	/* when current time is greater than irq time, call pic_request
 	   which will then point pic_itime to next interrupt */
 
-	if (((pic_itime[PIC_IRQ0]-pic_sys_time) < 0) && ((p->mode&0x40)==0)) {
-	  r_printf("PIT: pit_latch, pic_request IRQ0 mode 2/3\n");
-	  pic_request(PIC_IRQ0);
+	if (((p->mode&0x40)==0) && (pic_sys_time > pic_itime[PIC_IRQ0])) {
+	  if (pic_request(PIC_IRQ0)==PIC_REQ_OK)
+	   { r_printf("PIT: pit_latch, pic_request IRQ0 mode 2/3\n"); }
 	}
 	/* while current time is less than next irq time, ticks decrease;
-	   we need no modulo operation */
-	ticks = pic_itime[PIC_IRQ0] - pic_sys_time;
+         * ticks can go out of bounds or negative when the interrupt
+         * is lost or pending */
+	ticks = (pic_itime[PIC_IRQ0] - pic_sys_time) % p->cntr;
       } else {
 	ticks = p->cntr - (pic_sys_time % p->cntr);
       }
+
       if ((p->mode & 3)==3) {
         /* ticks is now a value which decreases from cntr to 0, and
            is greater than cntr/2 in the first half of the cycle */
@@ -328,9 +334,9 @@ static void pit_latch(int latch)
   if (p->mode & 0x40)
     p->mode = (p->mode & 7) | 0x80;
 
-#if 0 /* for debugging */
-  i_printf("pit_latch:  latched value 0x%4.4x for timer %d\n",
-	   p->read_latch, latch);
+#ifdef DEBUG_PIT
+  i_printf("PIT%d: ticks=%lx latch=%x pin=%d\n",latch,ticks,
+	p->read_latch,(p->outpin!=0));
 #endif
 }
 
@@ -342,10 +348,9 @@ Bit8u pit_inp(ioport_t port)
   int ret = 0;
   port -= 0x40;
 
-  if (port == 2) {
-    if (config.speaker == SPKR_NATIVE) return safe_port_in_byte(0x42);
-    pit_latch(port);
-    return(pit[port].outpin);
+  if ((port == 2) && (config.speaker == SPKR_NATIVE)) {
+	/* how could we come here if we defined PORT_FAST? */
+	return safe_port_in_byte(0x42);
   }
   else if (port == 1)
     i_printf("PIT:  someone is reading the CMOS refresh time?!?");
@@ -374,7 +379,7 @@ Bit8u pit_inp(ioport_t port)
       pit[port].read_latch = -1;
       break;
   }
-#if 0
+#ifdef DEBUG_PIT
   i_printf("PORT: pit_inp(0x%x) = 0x%x\n", port+0x40, ret);
 #endif
   return ret;
@@ -387,6 +392,7 @@ void pit_outp(ioport_t port, Bit8u val)
   if (port == 1)
     i_printf("PORT: someone is writing the CMOS refresh time?!?");
   else if (port == 2 && config.speaker == SPKR_NATIVE) {
+    /* how could we come here if we defined PORT_FAST? */
     safe_port_out_byte(0x42, val);
     return;
   }
@@ -394,6 +400,15 @@ void pit_outp(ioport_t port, Bit8u val)
   switch (pit[port].write_state) {
     case 0:
       pit[port].write_latch = pit[port].write_latch | ((val & 0xff) << 8);
+      /*
+       * TRICK: some graphics apps use the vertical retrace bit to
+       * calibrate themselves. AFAIK this is done by starting PIT#0
+       * with a value of 0 after detecting the sync, then reading it
+       * again at next sync pulse. In this special case, synchronizing
+       * the emulated vertical retrace with the PIT write yields some
+       * better timing results. It works under X and with emuretrace -- AV
+       */
+      if (pit[port].write_latch==0) t_vretrace = pic_sys_time;
       pit[port].write_state = 3;
       break;
     case 3:
@@ -407,7 +422,7 @@ void pit_outp(ioport_t port, Bit8u val)
       pit[port].write_latch = (val & 0xff) << 8;
       break;
     }
-#if 0
+#ifdef DEBUG_PIT
   i_printf("PORT: pit_outp(0x%x, 0x%x)\n", port+0x40, val);
 #endif
 
@@ -444,7 +459,7 @@ void pit_control_outp(ioport_t port, Bit8u val)
 {
   int latch = (val >> 6) & 0x03;
 
-#if 0
+#ifdef DEBUG_PIT
   i_printf("PORT: outp(0x43, 0x%x)\n",val);
 #endif
 
@@ -471,31 +486,32 @@ void pit_control_outp(ioport_t port, Bit8u val)
   switch (latch) {
     case 2:
       if (config.speaker == SPKR_NATIVE) {
+	/* hmmm.. timer 2 is in PORT_FAST mode, and there will be a
+	 * noticeable amount of time from this mode setting to the
+	 * timer freq settings... could this cause strange effects?
+	 */
         safe_port_out_byte(0x43, val);
 	break;
       }
       /* nobreak; */
     case 0:
     case 1:
-#ifdef MONOTON_MICRO_TIMING
-      pit[latch].mode        = (val >> 1) & 0x07;
-      if ((val & 4)==0) {      /* modes 0,1,4,5 */
-      /* set the time base for the counter - safety code for programs
-       * which use a non-periodical mode without reloading the counter
-       */
-       pit[latch].time.td = GETtickTIME(0);
-      }
-#endif
       if ((val & 0x30) == 0)
 	pit_latch(latch);
       else {
 	pit[latch].read_state  = (val >> 4) & 0x03;
 	pit[latch].write_state = (val >> 4) & 0x03;
-#ifndef MONOTON_MICRO_TIMING
 	pit[latch].mode        = (val >> 1) & 0x07;
+#ifdef MONOTON_MICRO_TIMING
+        if ((val & 4)==0) {      /* modes 0,1,4,5 */
+          /* set the time base for the counter - safety code for programs
+           * which use a non-periodical mode without reloading the counter
+           */
+          pit[latch].time.td = GETtickTIME(0);
+        }
 #endif
       }
-#if 0
+#ifdef DEBUG_PIT
       i_printf("PORT: writing outp(0x43, 0x%x)\n", val);
 #endif
       break;
@@ -610,12 +626,15 @@ void timer_int_engine(void)
 Bit8u spkr_io_read(ioport_t port) {
    if (port==0x61)  {
       if (config.speaker == SPKR_NATIVE)
+	 /* how could we come here if we defined PORT_FAST? */
          return port_safe_inb(0x61);
-      else
+      else {
 	 /* keep the connection between port 0x61 and PIT timer#2 */
+	 pit_latch(2);
          return ((*((Bit8u *)&pic_sys_time)&0x10) | /* or anything that toggles quick enough */
-		(pit_inp(0x42)? 0x20:0) |	/* outpin: 00 or 80 */
+		(pit[2].outpin? 0x20:0) |	/* outpin: 00 or 80 */
 		(port61&0xcf));
+      }
    }
    return 0xff;
 }
@@ -624,6 +643,7 @@ void spkr_io_write(ioport_t port, Bit8u value) {
    if (port==0x61) {
       switch (config.speaker) {
        case SPKR_NATIVE:
+	  /* how could we come here if we defined PORT_FAST? */
           port_safe_outb(0x61, value & 0x03);
           break;
       
