@@ -535,7 +535,7 @@ static Keymap_Scan_Type Dosemu_Ctrl_keys[] =
 
 /* This table is for translating the ESC key. */
 /* The entries with 0x00 are not going to be used by any sane person. */
-static Bit8u Ctrl_Char_Scan_Codes[32] =
+Bit8u Ctrl_Char_Scan_Codes[32] =
 {
   0x03,			/* ^@ */
   0x1E,			/* ^A */
@@ -1045,7 +1045,7 @@ static void slang_send_scancode(unsigned long lscan, unsigned char ch)
   }
 }
 
-static void n_do_slang_getkeys(void)
+void do_slang_getkeys(void)
 {
   SLang_Key_Type *key;
   unsigned long scan;
@@ -1319,7 +1319,7 @@ static void sl_print_error(char *str)
  * DANG_END_FUNCTION
  */
    
-static int n_slang_keyb_init(void) {
+int slang_keyb_init(void) {
   struct termios buf;
 
   k_printf("KBD: slang_keyb_init()\n");
@@ -1331,7 +1331,7 @@ static int n_slang_keyb_init(void) {
    
   if (tcgetattr(kbd_fd, &save_termios) < 0) {
     int ignore = 0;
-    if (errno == EINVAL) {
+    if (errno == EINVAL || errno == ENOTTY) {
       if ( (config.cardtype == CARD_NONE)
           || !strcmp(getenv("TERM"),"dumb")        /* most cron's have this */
           || !strcmp(getenv("TERM"),"none")        /* ... some have this */
@@ -1373,7 +1373,8 @@ static int n_slang_keyb_init(void) {
   cfgetispeed(&buf);
   cfgetospeed(&buf);
 #endif
-  if (tcsetattr(kbd_fd, TCSANOW, &buf) < 0 && errno != EINVAL) {
+  if (tcsetattr(kbd_fd, TCSANOW, &buf) < 0 
+         && errno != EINVAL && errno != ENOTTY) {
     error("slang_keyb_init(): Couldn't tcsetattr(kbd_fd,TCSANOW,...) !\n");
   }
 
@@ -1412,8 +1413,9 @@ static int n_slang_keyb_init(void) {
   return TRUE;
 }
 
-static void n_slang_keyb_close(void)  {
-   if (tcsetattr(kbd_fd, TCSAFLUSH, &save_termios) == -1  && errno != EINVAL) {
+void slang_keyb_close(void)  {
+   if (tcsetattr(kbd_fd, TCSAFLUSH, &save_termios) == -1
+            && errno != EINVAL && errno != ENOTTY) {
       error("slang_keyb_close(): failed to restore keyboard termios settings!\n");
    }
    if (save_kbd_flags != -1) {
@@ -1421,26 +1423,11 @@ static void n_slang_keyb_close(void)  {
    }
 }
 
-int slang_keyb_init(void);
-
 struct keyboard_client Keyboard_slang =  {
    "slang",                    /* name */
    slang_keyb_init,            /* init */
    NULL,                       /* reset */
-   n_slang_keyb_close,           /* close */
-   n_do_slang_getkeys            /* run */
+   slang_keyb_close,           /* close */
+   do_slang_getkeys            /* run */
 };
    
-int slang_keyb_init(void)
-{
-  extern o_slang_keyb_init(void);
-  extern void o_do_slang_getkeys(void);
-  extern void o_slang_keyb_close(void);
-  if (config.features[0]) {
-    Keyboard_slang.close = &o_slang_keyb_close;
-    Keyboard_slang.run = &o_do_slang_getkeys;
-    return o_slang_keyb_init();
-  }
-  return n_slang_keyb_init();
-}
-
