@@ -32,9 +32,9 @@ static Bit8u erasekey;
 
 /* unsigned char DOSemu_Slang_Escape_Character = 30; */
 
-extern char *DOSemu_Keyboard_Keymap_Prompt;
-extern int DOSemu_Terminal_Scroll;
-extern int DOSemu_Slang_Show_Help;
+const char *DOSemu_Keyboard_Keymap_Prompt = NULL;
+int DOSemu_Terminal_Scroll = 0;
+int DOSemu_Slang_Show_Help = 0;
 
 /*
  * The goal of the routines here is simple: to allow SHIFT, ALT, and
@@ -300,7 +300,7 @@ static Keymap_Scan_Type Normal_Map[] =
 
 /* This table is for translating the ESC key. */
 /* The entries with 0x00 are not going to be used by any sane person. */
-static Bit8u Ctrl_Char_Scan_Codes[32] =
+Bit8u Ctrl_Char_Scan_Codes[32] =
 {
   0x03,			/* ^@ */
   0x1E,			/* ^A */
@@ -642,7 +642,7 @@ static void slang_send_scancode(unsigned long lscan, unsigned char ch)
   }
 }
 
-void o_do_slang_getkeys(void)
+void do_slang_getkeys(void)
 {
   SLang_Key_Type *key;
   unsigned long scan;
@@ -835,7 +835,7 @@ void o_do_slang_getkeys(void)
  * DANG_END_FUNCTION
  */
    
-int o_slang_keyb_init(void) {
+int slang_keyb_init(void) {
   struct termios buf;
 
   k_printf("KBD: slang_keyb_init()\n");
@@ -885,7 +885,7 @@ int o_slang_keyb_init(void) {
   return TRUE;
 }
 
-void o_slang_keyb_close(void)  {
+void slang_keyb_close(void)  {
    if (tcsetattr(kbd_fd, TCSAFLUSH, &save_termios) == -1) {
       error("slang_keyb_close(): failed to restore keyboard termios settings!\n");
    }
@@ -893,3 +893,12 @@ void o_slang_keyb_close(void)  {
       fcntl(kbd_fd, F_SETFL, save_kbd_flags);
    }
 }
+
+struct keyboard_client Keyboard_slang =  {
+   "slang",                    /* name */
+   slang_keyb_init,            /* init */
+   NULL,                       /* reset */
+   slang_keyb_close,           /* close */
+   do_slang_getkeys            /* run */
+};
+   
