@@ -1408,7 +1408,7 @@ void mouse_move_absolute(int x, int y, int x_range, int y_range)
 
 	m_printf("mouse_move_absolute(%d, %d, %d, %d) -> %d %d \n",
 		 x, y, x_range, y_range, mouse.x, mouse.y);
-	m_printf("mouse_move_absolute dx:%d dy%d mickeyx%d mickeyy%d\n", 
+	m_printf("mouse_move_absolute dx:%d dy:%d mickeyx%d mickeyy%d\n", 
 		 dx, dy, mouse.mickeyx, mouse.mickeyy);
 	/*
 	 * update the event mask
@@ -1491,9 +1491,10 @@ mouse_delta(int event)
 void
 mouse_event()
 {
-  if (mouse.mask & mouse_events && mouse.cs && mouse.ip) {
+  if (mouse.mask & mouse_events && (mouse.cs || mouse.ip)) {
     if(in_dpmi && !in_dpmi_dos_int 
-          && !((mouse.cs == DPMI_SEG) && ((void *)(Bit32u)mouse.ip == DPMI_mouse_callback)))
+          && !((mouse.cs == DPMI_SEG) && 
+          ((Bit32u)mouse.ip == (DPMI_OFF+HLT_OFF(DPMI_mouse_callback)))))
                   fake_pm_int();
 
     /* push iret frame on _SS:_SP. At F000:20F7 (bios.S) we get an
@@ -1524,7 +1525,7 @@ mouse_event()
 
     if (in_dpmi && 
 	(REG(cs) == DPMI_SEG) 
-	&& ((void *)REG(eip) == DPMI_mouse_callback))
+	&& (REG(eip) == (DPMI_OFF+HLT_OFF(DPMI_mouse_callback))))
 	run_pm_mouse();
     
     m_printf("MOUSE: event %d, x %d ,y %d, mx %d, my %d, b %x\n",
