@@ -1,3 +1,9 @@
+/* 
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 /*
  *
  *     Packet driver emulation for the Linux DOS emulator.
@@ -30,19 +36,24 @@
 #include "dosio.h"
 #include "memory.h"
 #include "int.h"
+#include "bios.h"
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <sys/time.h>
-#include <asm/byteorder.h>
-#if __GLIBC__ > 1
-#include <netinet/if_ether.h>
+#if GLIBC_VERSION_CODE >= 2000
+  #if GLIBC_VERSION_CODE == 2000
+    #include <asm/byteorder.h>
+  #endif
+  #include <netinet/if_ether.h>
 #else
-#include <linux/if_ether.h>
+  #include <asm/byteorder.h>
+  #include <linux/if_ether.h>
 #endif
 #include "libpacket.h"
 #include "dosnet.h"
+#include "dosemu_select.h"
 
 /* flag to activate use of pic by packet driver */
 #if 0
@@ -68,7 +79,7 @@ static int pktdrvr_installed;
 struct pkt_type {
     int handle;
     int pkt_type_len;
-    char pkt_type[MAX_PKT_TYPE_SIZE];
+    unsigned char pkt_type[MAX_PKT_TYPE_SIZE];
     int count;       /* To sort the array based on counts of packet type */
 } pkt_type_array[MAX_HANDLE]; 
 int max_pkt_type_array=0;
@@ -107,7 +118,6 @@ static char devname[10] = "eth0";   /* linux device name */
 /* various declarations for interfacing with the packet driver code in
    bios.S */
 
-extern void bios_f000();
 extern void PKTDRV_size();
 extern void PKTDRV_handle();
 extern void PKTDRV_receiver();
@@ -157,7 +167,7 @@ pkt_init(int vec)
 
     /* hook the interrupt vector by pointing it into the magic table */
     SETIVEC(vec, PKTDRV_SEG, PKTDRV_OFF);
-    memcpy(MK_PTR(PKTDRV_signature), "PKT DRVR", 8);
+    strcpy(MK_PTR(PKTDRV_signature), "PKT DRVR");
     
     /* fill other global data */
 
@@ -571,7 +581,7 @@ Insert_Type(int handle, int pkt_type_len, char *pkt_type)
     memcpy(&pkt_type_array[max_pkt_type_array].pkt_type, 
 	   pkt_type, pkt_type_len);
     for(i=0; i<pkt_type_len; i++) 
-	pd_printf(" --%.2x--", pkt_type_array[max_pkt_type_array].pkt_type[i]);
+	pd_printf(" %.2x", pkt_type_array[max_pkt_type_array].pkt_type[i]);
     pd_printf("\n");
     pd_printf("Succeeded: inserted at %d\n", max_pkt_type_array);
     max_pkt_type_array++;
@@ -645,7 +655,6 @@ pkt_check_receive(int timeout)
 	/* anything ready? */
 	if (select(max_pkt_fd,&readset,NULL,NULL,&tv) <= 0)
 	    return 0;
-	pd_printf("select returned\n");
 	if(FD_ISSET(pkt_fd, &readset)) 
 	    fd = pkt_fd;
 	else if(FD_ISSET(pkt_broadcast_fd, &readset)) 
