@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -11,7 +11,7 @@
  *
  * Functions used to get a monoton, 64-bit time value either from the
  * kernel (with gettimeofday()) or from the CPU hi-res timer
- * Maintainer: vignani@mbox.vol.it (Alberto Vignani)
+ * Maintainer: vignani@mail.tin.it (Alberto Vignani)
  * 
  * SIDOC_END_MODULE
  */
@@ -93,8 +93,10 @@ static hitimer_t rawC4time(void)
 {
 #ifdef HAVE_GETTIMEOFDAY
   struct timeval tv;
-  gettimeofday(&tv, NULL);	/* takes 30us on a P5-150 */
-  return ((hitimer_t)tv.tv_sec*1000000 + (hitimer_t)tv.tv_usec);
+  gettimeofday(&tv, NULL);	/* took 30us on a P5-150 in 1997, now
+  				 * takes 1-2us on a K6-300 under 2.1.126 */
+  /* unsigned is necessary to tell gcc not to sign extend the tv_ fields */
+  return ((hitimer_t)((unsigned)tv.tv_sec) * 1000000 + (unsigned)tv.tv_usec);
 #else
 #error Cannot get time
 #endif
@@ -171,6 +173,7 @@ hitimer_t GETusSYSTIME(void)
 
 void get_time_init (void)
 {
+  extern int kernel_version_code;
   if ((config.realcpu > CPU_486) && config.rdtsc) {
     /* we are here if: a 586/686 was detected at startup, we are not
      * on a SMP machine and the user didn't say 'rdtsc off'. But
@@ -185,7 +188,12 @@ void get_time_init (void)
      * 'rdtsc off' into config file */
     RAWcpuTIME = rawC4time;
     GETcpuTIME = getC4time;
-    g_printf("TIMER: using gettimeofday\n");
+    if (config.realcpu) {
+      if (kernel_version_code < 0x2017e)
+        g_printf("TIMER: using gettimeofday\n");
+      else
+        g_printf("TIMER: using new gettimeofday with microsecond resolution\n");
+    }
   }
   ZeroTimeBase.td = RAWcpuTIME();
 }
@@ -194,10 +202,10 @@ void get_time_init (void)
 /* --------------------------------------------------------------------- */
 
 
-int stop_cputime (void)
+int stop_cputime (int quiet)
 {
   if (cpu_time_stop) return 1;
-  dbug_printf("STOP TIME\n");
+  if (!quiet) dbug_printf("STOP TIME\n");
   StopTimeBase = RAWcpuTIME();
   LastTimeRead = StopTimeBase - ZeroTimeBase.td;
   cpu_time_stop = 1;
@@ -205,10 +213,10 @@ int stop_cputime (void)
 }
 
 
-int restart_cputime (void)
+int restart_cputime (int quiet)
 {
   if (!cpu_time_stop) return 1;
-  dbug_printf("RESTART TIME\n");
+  if (!quiet) dbug_printf("RESTART TIME\n");
   ZeroTimeBase.td += (RAWcpuTIME() - StopTimeBase);
   cpu_time_stop = 0;
   return 0;
@@ -217,18 +225,17 @@ int restart_cputime (void)
 
 /* --------------------------------------------------------------------- */
 
-int getmhz(void)
+static int getmhz(void)
 {
 	struct timeval tv1,tv2;
 	hitimer_t a,b;
-	int i, rawspeed;
-	volatile int j;
 
 	gettimeofday(&tv1, NULL);
 	__asm__ __volatile__ ("rdtsc"
 		:"=a" (((unsigned long*)&a)[0]),
 		 "=d" (((unsigned long*)&a)[1]));
-	for (j=0; j<10000000; j++);	/* 500ms on a P5-100 */
+/*	for (j=0; j<10000000; j++);*/	/* 500ms on a P5-100 */
+	usleep(50000);
 	gettimeofday(&tv2, NULL);
 	__asm__ __volatile__ ("rdtsc"
 		:"=a" (((unsigned long*)&b)[0]),
@@ -236,23 +243,7 @@ int getmhz(void)
 	b -= a;
 	a = (tv2.tv_sec*1000000 + tv2.tv_usec) - 
 	    (tv1.tv_sec*1000000 + tv1.tv_usec);
-	/* how to round? standard clock speeds are usually the
-	 * multiples of 30 and 33.333, plus 75. Some people
-	 * use non-standard clock values (like 208) by fiddling with
-	 * the motherboard jumpers: but we hope they are smart enough
-	 * to read the READMEs and set their right speed in dosemu.conf.
-	 * OTOH, I'm not smart enough to come up with a better
-	 * algorithm  --AV
-	 */
-	rawspeed = (int)(b/a);
-	for (i=67; i<=300; i+=33) {
-	  if ((i%10)==6) i++;	/* 133,167,200... */
-	  if (abs(rawspeed-i)<3) return i;
-	}
-	for (i=60; i<=300; i+=(i>=90? 30:15)) {
-	  if (abs(rawspeed-i)<3) return i;
-	}
-	return rawspeed;
+	return (int)((b*4096)/a);
 }
 
 /*
@@ -264,7 +255,7 @@ int getmhz(void)
 int bogospeed(unsigned long *spus, unsigned long *sptick)
 {
 	boolean first=1;
-	int mlt=0, dvs=0;
+	int mlt, dvs;
 
 	if (config.realcpu < CPU_586) {
 		fprintf(stderr,"You can't access 586 features on CPU=%d\n",
@@ -272,22 +263,8 @@ int bogospeed(unsigned long *spus, unsigned long *sptick)
 		exit(1);
 	}
 	if (!first) return 0;
-#if 0
-	{ char *p;
-	/* user-defined speed */
-	if ((p=getenv("CPUSPEED"))!=NULL) {
-	  int v1;
-	  if (sscanf(p,"%d",&v1)==1) {
-	    if ((v1>60) && (v1<=300)) {
-	      mlt = 1; dvs = v1;
-	    }
-	  }
-	}}
-#endif
-	if (!mlt) {
-	/* last resort - do it yourself */
-	  mlt = 1; dvs = getmhz();
-	}
+
+	mlt = 4096; dvs = getmhz();
 
 	/* speed division factor to get 1us from CPU clocks - for
 	 * details on fast division see timers.h */
@@ -299,7 +276,8 @@ int bogospeed(unsigned long *spus, unsigned long *sptick)
 #ifdef X86_EMULATOR
 	config.emuspeed = dvs/mlt;
 #endif
-	fprintf (stderr,"CPU speed set to %d/%d MHz\n",dvs,mlt);
+	fprintf (stderr,"CPU speed set to %d MHz\n",(dvs/mlt));
+/*	fprintf (stderr,"CPU speed factors %ld,%ld\n",*spus,*sptick); */
 	first = 0;
 	return 0;
 }
