@@ -71,6 +71,7 @@
 #include "ipx.h"                /* TRB - add support for ipx */
 #include "keymaps.h"
 #include "bitops.h"
+#include "cpu-emu.h"
 
 #include "video.h"
 #if X_GRAPHICS
@@ -301,6 +302,14 @@ void vm86_GP_fault(void)
        /* set VIF (only if necessary) */
     if (REG(eflags) & IF_MASK) REG(eflags) |= VIF_MASK;
 #endif /* not USE_NEW_INT */
+#if defined(X86_EMULATOR) && defined(SKIP_EMU_VBIOS)
+    if ((config.cpuemu>1) && (lina == (unsigned char *) CPUEMUI10_ADD)) {
+      e_printf("EMU86: HLT at int10 end\n");
+      LWORD(eip) += 1;	/* simply skip, so that we go back to emu mode */
+      break;
+    }
+    else
+#endif
           /* return with STI if VIP was set from run_dpmi; this happens
            * if pic_count is >0 and the VIP flag in dpmi_eflags was on
            */
@@ -308,6 +317,12 @@ void vm86_GP_fault(void)
       pic_iret();
     }
 
+    else if (lina == (unsigned char *) CBACK_ADD) {
+      /* we are back from a callback routine */
+      static void callback_return(void);
+      callback_return();
+    }
+
     else if (lina == (unsigned char *) XMSTrap_ADD) {
       LWORD(eip) += 2;  /* skip halt and info byte to point to FAR RET */
       xms_control();
@@ -321,7 +336,7 @@ void vm86_GP_fault(void)
     }
 
     else {
-#if 1
+#ifndef SKIP_EMU_VBIOS
       error("HLT requested: lina=%p!\n", lina);
       show_regs(__FILE__, __LINE__);
 #if 0
@@ -348,6 +363,7 @@ op0ferr:
 #ifdef USE_MHPDBG
     mhp_debug(DBG_GPF, 0, 0);
 #endif
+    d.general=1;
     error("general protection at %p: %x\n", lina,*lina);
     show_regs(__FILE__, __LINE__);
     show_ints(0, 0x33);
@@ -437,50 +453,23 @@ run_vm86(void)
 #endif
     /* FIXME: this needs to be clarified and rewritten */
 
-#if 0
-#define PFLAG(f)  if (REG(eflags)&(f)) k_printf(#f" ")
-
-  k_printf("FLAGS BEFORE: ");
-  PFLAG(CF);
-  PFLAG(PF);
-  PFLAG(AF);
-  PFLAG(ZF);
-  PFLAG(SF);
-  PFLAG(TF);
-  PFLAG(IF);
-  PFLAG(DF);
-  PFLAG(OF);
-  PFLAG(NT);
-  PFLAG(RF);
-  PFLAG(VM);
-  PFLAG(AC);
-  PFLAG(VIF);
-  PFLAG(VIP);
-  k_printf(" IOPL: %u\n", (unsigned) ((vflags & IOPL_MASK) >> 12));
+    if (
+#ifdef X86_EMULATOR
+	(d.emu>1)||
 #endif
+	(d.general>3)) {
+	dbug_printf ("DO_VM86,  cs=%04x:%04x ss=%04x:%04x f=%08x\n",
+		_CS, _EIP, _SS, _SP, _EFLAGS);
+	if (d.general>8)
+	    dbug_printf ("ax=%04x bx=%04x ss=%04x sp=%04x bp=%04x\n"
+	      		 "           cx=%04x dx=%04x ds=%04x cs=%04x ip=%04x\n"
+	      		 "           si=%04x di=%04x es=%04x flg=%08x\n",
+			_AX, _BX, _SS, _SP, _BP, _CX, _DX, _DS, _CS, _IP,
+			_SI, _DI, _ES, _EFLAGS);
+    }
 
     retval = DO_VM86(&vm86s);
 
-#if 0
-  k_printf("FLAGS AFTER: ");
-  PFLAG(CF);
-  PFLAG(PF);
-  PFLAG(AF);
-  PFLAG(ZF);
-  PFLAG(SF);
-  PFLAG(TF);
-  PFLAG(IF);
-  PFLAG(DF);
-  PFLAG(OF);
-  PFLAG(NT);
-  PFLAG(RF);
-  PFLAG(VM);
-  PFLAG(AC);
-  PFLAG(VIF);
-  PFLAG(VIP);
-  k_printf(" IOPL: %u\n", (unsigned) ((vflags & IOPL_MASK) >> 12));
-#endif
-
 #ifdef USE_NEW_INT
   /* sync the pic interrupt state with the flags && sync VIF & IF */
     if (_EFLAGS & VIF) {
@@ -495,6 +484,20 @@ run_vm86(void)
       g_printf("BUG: flags changed to %08x\n",_EFLAGS);
       _EFLAGS &= ~(AC|ID);
     }
+    if (
+#ifdef X86_EMULATOR
+	(d.emu>1)||
+#endif
+	(d.general>3)) {
+	dbug_printf ("RET_VM86, cs=%04x:%04x ss=%04x:%04x f=%08x ret=0x%x\n",
+		_CS, _EIP, _SS, _SP, _EFLAGS, retval);
+	if (d.general>8)
+	    dbug_printf ("ax=%04x bx=%04x ss=%04x sp=%04x bp=%04x\n"
+	      		 "           cx=%04x dx=%04x ds=%04x cs=%04x ip=%04x\n"
+	      		 "           si=%04x di=%04x es=%04x flg=%08x\n",
+			_AX, _BX, _SS, _SP, _BP, _CX, _DX, _DS, _CS, _IP,
+			_SI, _DI, _ES, _EFLAGS);
+    }
 
     in_vm86 = 0;
     switch VM86_TYPE
@@ -532,6 +535,19 @@ run_vm86(void)
 	}
 
     handle_signals();
+
+    { /* catch user hooks here */
+    	extern int uhook_fdin;
+    	extern void uhook_poll(void);
+    	if (uhook_fdin != -1) uhook_poll();
+    }
+
+    /* here we include the hooks to possible plug-ins */
+    #define VM86_RETURN_VALUE retval
+    #include "plugin_poll.h"
+    #undef VM86_RETURN_VALUE
+
+
 #ifdef USE_MHPDBG  
     if (mhpdbg.active) mhp_debug(DBG_POLL, 0, 0);
 #endif
@@ -559,4 +575,51 @@ run_vm86(void)
 /* @@@ MOVE_END @@@ 49152 */
 
 
+static int callback_level = 0;
+
+static void callback_return(void)
+{
+	callback_level--;
+}
+
+/*
+ * do_call_back() calls a 16-bit DOS routine with a far call.
+ * NOTE: It does _not_ save any of the vm86 registers except old cs:ip !!
+ *       The _caller_ has to do this.
+ *
+ * FIXME: This does not yet handle DPMI stuff, it should not be called
+ *        while in_dpmi is active.
+ */
+void do_call_back(Bit32u codefarptr)
+{
+	unsigned char * ssp;
+	unsigned long sp;
+	Bit16u oldcs, oldip;
+	int level;
+
+	/* we push the address of our HLT place in the bios
+	 * as return address on the stack and run vm86 mode.
+	 * Hence we get awaoken by sigsegv, whih in return calls
+	 * callback_return() above, which then decreases callback_level
+	 * ... and then we return from here
+	 */
+	ssp = (unsigned char *)(LWORD(ss)<<4);
+	sp = (unsigned long) LWORD(esp);
+	pushw(ssp, sp, CBACK_SEG);	/* push our return cs:ip */
+	pushw(ssp, sp, CBACK_OFF);
+	LWORD(esp) = (LWORD(esp) - 4) & 0xffff;
+	oldcs = REG(cs);		/* save old cs:ip */
+	oldip = LWORD(eip);
+	REG(cs) = codefarptr >>16;	/* far jump to the vm86(DOS) routine */
+	LWORD(eip) = codefarptr & 0xffff;
+
+        level = callback_level++;
+        while (callback_level > level) {
+		run_vm86();
+		serial_run();
+        }
+	/* ... and back we are */
+	REG(cs) = oldcs;
+	LWORD(eip) = oldip;
+}
 
