@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -43,7 +43,7 @@
  *
  * 1997/06/15: Added quit a few definitions related to the new global
  * variable `vga' and the rewrite of vgaemu.c.
- * -- sw (Steffen.Winterfeldt@itp.uni-leipzig.de)
+ * -- sw (Steffen Winterfeldt <wfeldt@suse.de>)
  *
  * 1997/07/08: Integrated vgaemu_inside.h; there was no file that included
  * only either of vgaemu.h/vgaemu_inside.h anyway.
@@ -90,58 +90,74 @@
 #define P32		19
 #define PL1		20
 #define TEXT_MONO	21
+#define PL2		22
 
 
 /*
  * Definition of the port addresses.
  */
 
-#define CRTC_BASE           0x3d4
-#define CRTC_INDEX          CRTC_BASE
-#define CRTC_DATA           CRTC_BASE+0x01
+#define CRTC_INDEX_MONO		0x3b4
+#define CRTC_DATA_MONO		0x3b5
 
-#define INPUT_STATUS_1      0x3da
-#define FEATURE_CONTROL_1   INPUT_STATUS_1
+#define HERC_MODE_CTRL		0x3b8
 
-#define VGA_BASE            0x3c0
+#define INPUT_STATUS_1_MONO	0x3ba
+#define FEATURE_CONTROL_W_MONO	0x3ba
 
-#define ATTRIBUTE_BASE      VGA_BASE
-#define ATTRIBUTE_INDEX     ATTRIBUTE_BASE
-#define ATTRIBUTE_DATA      ATTRIBUTE_BASE+0x01
+#define HERC_CFG_SWITCH		0x3bf
 
-#define INPUT_STATUS_0      VGA_BASE+0x02
-#define MISC_OUTPUT_0       VGA_BASE+0x02
-#define SUBSYSTEM_ENABLE    VGA_BASE+0x03
 
-#define SEQUENCER_BASE      VGA_BASE+0x04
-#define SEQUENCER_INDEX     SEQUENCER_BASE
-#define SEQUENCER_DATA      SEQUENCER_BASE+0x01
+#define VGA_BASE		0x3c0
 
-#define GFX_CTRL_BASE       VGA_BASE+0x0e
+#define ATTRIBUTE_INDEX		0x3c0
+#define ATTRIBUTE_DATA		0x3c1
 
-#define DAC_BASE            VGA_BASE+0x06
-#define DAC_PEL_MASK        DAC_BASE
-#define DAC_STATE           DAC_BASE+0x01
-#define DAC_READ_INDEX      DAC_BASE+0x01
-#define DAC_WRITE_INDEX     DAC_BASE+0x02
-#define DAC_DATA            DAC_BASE+0x03
+#define INPUT_STATUS_0		0x3c2
+#define MISC_OUTPUT_W		0x3c2
+#define SUBSYSTEM_ENABLE	0x3c3		/* currently unused */
 
-#define FEATURE_CONTROL_0   VGA_BASE+0x0a
-#define GRAPHICS_2_POSITION FEATURE_CONTROL
-#define MISC_OUTPUT_1       VGA_BASE+0x0c
-#define GRAPHICS_1_POSITION MISC_OUTPUT_1
+#define SEQUENCER_INDEX		0x3c4
+#define SEQUENCER_DATA		0x3c5
+
+#define DAC_PEL_MASK		0x3c6
+#define DAC_STATE		0x3c7
+#define DAC_READ_INDEX		0x3c7
+#define DAC_WRITE_INDEX		0x3c8
+#define DAC_DATA		0x3c9
+
+#define FEATURE_CONTROL_R	0x3ca
+#define MISC_OUTPUT_R		0x3cc
+
+#define GFX_INDEX		0x3ce
+#define GFX_DATA		0x3cf
+
+
+#define CRTC_INDEX		0x3d4
+#define CRTC_DATA		0x3d5
+
+#define INPUT_STATUS_1		0x3da
+#define FEATURE_CONTROL_W	0x3da
+
+
+/*
+ *
+ */
+
+#define CFG_SEQ_ADDR_MODE	1
+#define CFG_CRTC_ADDR_MODE	2
 
-#define GRAPHICS_BASE       VGA_BASE+0x0e
-#define GRAPHICS_INDEX      GRAPHICS_BASE
-#define GRAPHICS_DATA       GRAPHICS_BASE+0x01
 
 /*
  * A DAC entry. Note that r, g, b may be 6 or 8 bit values,
  * depending on the current DAC width (stored in vga.dac.bits).
+ *
+ * Note that the 'index'-entry is (ab)used internally to hold the
+ * dirty flag.
  */
 
 typedef struct {
-  unsigned char r, g, b;	/* red, green, blue */
+  unsigned char index, r, g, b;		/* index, red, green, blue */
 } DAC_entry;
 
 
@@ -151,11 +167,11 @@ typedef struct {
  */
 
 typedef struct {
-  int mode;			/* video mode number actually used to set the mode (incl. bit 15, 14, 7) */
-  int VESA_mode;		/* VESA mode number */
+  int VGA_mode;			/* VGA mode number (-1 = none) */
+  int VESA_mode;		/* VESA mode number (-1 = none) */
   int mode_class;		/* mode class (TEXT/GRAPH) */
   int type;			/* used memory model */
-  int color_bits;		/* bits per colors */
+  int color_bits;		/* bits per color */
   int width, height;		/* resolution in pixels */
   int text_width, text_height;	/* resolution in characters */
   int char_width, char_height;	/* size of the character box */
@@ -164,30 +180,6 @@ typedef struct {
 } vga_mode_info;
 
 
-/*
- * Type of indexed registers.
- */
-
-enum register_type {
-  reg_read_write,		/* value read == value written */
-  reg_read_only,		/* write to this type of register is undefined */
-  reg_write_only,		/* read from this type of register returns 0 */
-  reg_double_function		/* value read != value written */
-};
-
-
-/*
- * Indexed register data structure.
- */
-
-typedef struct {
-  unsigned char read;		/* value read */
-  unsigned char write;		/* value written */
-  int type;			/* register type, choose one of enum register_type */
-  int dirty;			/* register changed? */
-} indexed_register;
-
-
 /*
  * Describes the type of display VGAEmu is attached to.
  */
@@ -228,6 +220,8 @@ typedef struct {
 				   does _not_ indicate a change of the display start */
   unsigned dac		: 1;	/* DAC reconfig (e.g. DAC width changed) */
   unsigned power	: 1;	/* display power state changed */
+
+  unsigned re_init	: 1;	/* re-initialize video mode according to vga struct */
 } vga_reconfig_type;
 
 
@@ -237,10 +231,9 @@ typedef struct {
  * We need probably only 3 mappings.
  * 0: 0xa000/0xb800 for banked graphics/text (vga.mem.bank refers to this mapping)
  * 1: LFB
- * 2: 0xa000/0xb000/0xb800 for fonts/extra text if necessary (not used yet)
  */
 
-#define VGAEMU_MAX_MAPPINGS	3
+#define VGAEMU_MAX_MAPPINGS	2
 
 #define VGAEMU_MAP_BANK_MODE	0
 #define VGAEMU_MAP_LFB_MODE	1
@@ -262,46 +255,121 @@ typedef struct {
   unsigned size;			/* size of memory in bytes */
   unsigned pages;			/* dto in pages */
   int fd;				/* file descriptor for "/proc/self/mem" */
+  unsigned scratch_page;		/* for unmapped areas */
   vga_mapping_type map[VGAEMU_MAX_MAPPINGS];	/* all the mappings */
   unsigned bank_pages;			/* size of a bank in pages */
   unsigned bank;			/* selected bank */
   unsigned char *dirty_map;		/* 1 == dirty */
   int planes;				/* 4 for PL4 and ModeX, 1 otherwise */
+  int plane_pages;			/* pages per plane  */
   int write_plane;			/* 1st (of up to 4) planes */
   int read_plane;
 } vga_mem_type;
 
 
 /*
- * All DAC data. (in future -- sw)
+ * All DAC data.
  */
 
 typedef struct {
-  unsigned bits;
+  unsigned bits;			/* DAC bits, usually 6 or 8 */
+  char pel_index;			/* 'r', 'g', 'b' */
+  unsigned char pel_mask;
+  unsigned char state;
+  unsigned char read_index;
+  unsigned char write_index;
+  DAC_entry rgb[0x100];			/* Note: rgb.index holds dirty flag! */
 } vga_dac_type;
 
 
 /*
- * All CRTC data.
+ * All Attribute Controller data.
  */
 
-#define CRTC_MAX_INDEX 255	/* number of emulated CRTC registers - 1 */
-
+#define ATTR_MAX_INDEX 0x14		/* 21 registers */
 
 typedef struct {
-  indexed_register data[CRTC_MAX_INDEX + 1];
-  unsigned index;
+  unsigned flipflop;			/* index/data */
+  unsigned char index;			/* index reg (5 bit) */
+  unsigned char cpu_video;		/* pal regs unreadable if 1 */
+  unsigned char data[ATTR_MAX_INDEX + 1];
+  unsigned char dirty[ATTR_MAX_INDEX + 1];
+} vga_attr_type;
+
+
+/*
+ * All CRT Controller data.
+ */
+
+#define CRTC_MAX_INDEX 0x18		/* 25 registers */
+
+typedef struct {
+  unsigned addr_mode;
+  unsigned cursor_location;
+  unsigned char index;
+  unsigned char data[CRTC_MAX_INDEX + 1];
+  unsigned char dirty[CRTC_MAX_INDEX + 1];
 } vga_crtc_type;
 
 
 /*
- * All Sequencer data. (in future -- sw)
+ * All Sequencer data.
+ */
+
+#define SEQ_MAX_INDEX 0x0f		/* 16 registers */
+
+typedef struct {
+  unsigned addr_mode;
+  unsigned char map_mask;
+  unsigned char mode_ctrl_1_bak, mode_ctrl_2_bak;
+  unsigned char mode;
+  unsigned char index;
+  unsigned char data[SEQ_MAX_INDEX + 1];
+} vga_seq_type;
+
+
+/*
+ * All Graphics Controller data.
+ */
+
+#define GFX_MAX_INDEX 0x08		/* 9 registers */
+
+typedef struct {
+  unsigned char index;
+  unsigned char data[GFX_MAX_INDEX + 1];
+} vga_gfx_type;
+
+
+/*
+ * Miscellaneous VGA registers.
  */
 
 typedef struct {
-  unsigned char chain4;
-  unsigned char map_mask;
-} vga_seq_type;
+  unsigned char misc_output, feature_ctrl;
+} vga_misc_type;
+
+
+/*
+ * Hercules Emulation specific data.
+ */
+
+typedef struct {
+  unsigned char cfg_switch, mode_ctrl;
+} vga_herc_type;
+
+
+/*
+ * Some VGA configuration info.
+ */
+
+typedef struct {
+  unsigned mono_support		: 1;	/* set if we support mono modes */
+  unsigned standard		: 1;	/* set for modes that are register compatible */
+  unsigned mono_port		: 1;	/* set if we use 0x3b0... instead of 0x3d0... */
+  unsigned video_off		: 4;	/* != 0 if no video signal is generated;
+  					 * bit 0-3: due to attr/seq/crtc/herc
+  					 */
+} vga_config_type;
 
 
 /*
@@ -309,11 +377,13 @@ typedef struct {
  */
 
 typedef struct {
-  int mode;				/* the video mode number */
-  int VESA_mode;
+  int mode;				/* video mode number actually used to
+  					   set the mode (incl. bit 15, 14, 7) */
+  int VGA_mode;				/* VGA mode number (-1 = none) */
+  int VESA_mode;			/* VESA mode number (-1 = none) */
   int mode_class;			/* TEXT, GRAPH */
   int mode_type;			/* TEXT, CGA, PL4, ... */
-  vga_mode_info *mode_info;		/* ptr into vga_mode_table */
+  vga_config_type config;		/* config info */
   vga_reconfig_type reconfig;		/* indicate when essential things have changed */
   int width;				/* in pixels */
   int height;				/* dto */
@@ -321,13 +391,20 @@ typedef struct {
   int text_width;
   int text_height;
   int char_width, char_height;		/* character cell size */
+  int color_bits;			/* color size */
   int pixel_size;			/* bits / pixel (including reserved bits) */
+  int buffer_seg;			/* segment for banked modes */
   int display_start;			/* offset for the 1st pixel */
   int power_state;			/* display power state (cf. VBE functions) */
+  int color_modified;			/* set if some palette/dac data have been changed */
   vga_mem_type mem;
   vga_dac_type dac;
+  vga_attr_type attr;
   vga_crtc_type crtc;
   vga_seq_type seq;
+  vga_gfx_type gfx;
+  vga_misc_type misc;
+  vga_herc_type herc;
 } vga_type;
 
 
@@ -381,12 +458,17 @@ int vga_emu_fault(struct sigcontext *, int);
 int vga_emu_init(vgaemu_display_type *);
 void vga_emu_done(void);
 int vga_emu_update(vga_emu_update_type *);
+int vgaemu_switch_plane(unsigned);
 int vga_emu_switch_bank(unsigned);
 vga_mode_info *vga_emu_find_mode(int, vga_mode_info *);
 int vga_emu_setmode(int, int, int);
+int vgaemu_map_bank(void);
+int vga_emu_set_textsize(int, int);
 void dirty_all_video_pages(void);
 int vga_emu_set_text_page(unsigned, unsigned);
-
+void dirty_all_vga_colors(void);
+int changed_vga_colors(DAC_entry *);
+void vgaemu_adj_cfg(unsigned, unsigned);
 
 /*
  * Functions defined in env/video/vesa.c.
@@ -394,7 +476,6 @@ int vga_emu_set_text_page(unsigned, unsigned);
 
 void vbe_init(vgaemu_display_type *);
 void do_vesa_int(void);
-int vesa_emu_fault(struct sigcontext_struct *scp);
 
 
 /*
@@ -402,20 +483,21 @@ int vesa_emu_fault(struct sigcontext_struct *scp);
  */
 
 void DAC_init(void);
-void DAC_dirty_all(void);
-void DAC_get_entry(DAC_entry *, unsigned char);
-void DAC_read_entry(DAC_entry *, unsigned char);
-int DAC_get_dirty_entry(DAC_entry *);
+void DAC_set_width(unsigned);
+void DAC_get_entry(DAC_entry *);
 void DAC_set_entry(unsigned char, unsigned char, unsigned char, unsigned char);
-unsigned char DAC_get_pel_mask(void);
-unsigned char DAC_get_state(void);
+void DAC_rgb2gray(unsigned char);
+
+void DAC_set_read_index(unsigned char);
+void DAC_set_write_index(unsigned char);
 
-inline void DAC_set_read_index(unsigned char);
-inline void DAC_set_write_index(unsigned char);
 unsigned char DAC_read_value(void);
 void DAC_write_value(unsigned char);
-inline void DAC_set_pel_mask(unsigned char);
-void DAC_set_width(unsigned);
+
+unsigned char DAC_get_pel_mask(void);
+void DAC_set_pel_mask(unsigned char);
+
+unsigned char DAC_get_state(void);
 
 
 /*
@@ -423,10 +505,13 @@ void DAC_set_width(unsigned);
  */
 
 void Attr_init(void);
-void Attr_write_value(unsigned char);
+unsigned char Attr_get_entry(unsigned char);
+void Attr_set_entry(unsigned char, unsigned char);
+
 unsigned char Attr_read_value(void);
-inline unsigned char Attr_get_index(void);
-unsigned char Attr_get_input_status_1(void);
+void Attr_write_value(unsigned char);
+
+unsigned char Attr_get_index(void);
 
 
 /*
@@ -451,4 +536,45 @@ void CRTC_write_value(unsigned char);
 unsigned char CRTC_read_value(void);
 
 
+/*
+ * Functions defined in env/video/gfxemu.c.
+ */
+
+void GFX_init(void);
+void GFX_set_index(unsigned char);
+unsigned char GFX_get_index(void);
+void GFX_write_value(unsigned char);
+unsigned char GFX_read_value(void);
+
+
+/*
+ * Functions defined in env/video/miscemu.c.
+ */
+
+void Misc_init(void);
+void Misc_set_misc_output(unsigned char);
+unsigned char Misc_get_misc_output(void);
+void Misc_set_feature_ctrl(unsigned char);
+unsigned char Misc_get_feature_ctrl(void);
+unsigned char Misc_get_input_status_0(void);
+unsigned char Misc_get_input_status_1(void);
+
+
+/*
+ * Functions defined in env/video/hercemu.c.
+ */
+
+void Herc_init(void);
+void Herc_set_cfg_switch(unsigned char);
+void Herc_set_mode_ctrl(unsigned char);
+unsigned char Herc_get_mode_ctrl(void);
+
+
+/*
+ * Functions defined in env/video/instremu.c.
+ */
+
+unsigned instr_len(unsigned char *);
+
+
 #endif	/* !defined __VGAEMU_H */
