@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -35,7 +35,7 @@
 #include <sys/stat.h>
 #include <time.h>
 #include <unistd.h>
-#if __GLIBC__ > 1
+#if GLIBC_VERSION_CODE >= 2000
 #include <errno.h>
 #endif
 
